<?php

final class ThemeRainPortfolio {

	public function __construct() {
		register_activation_hook( __FILE__, array( $this, 'plugin_activation' ) );
		add_action( 'init', array( $this, 'portfolio_init' ) );
		add_action( 'admin_init', array( $this, 'portfolio_custom_slug' ) );
		add_filter( 'manage_edit-project_columns', array( $this, 'add_thumbnail_column' ), 10, 1 );
		add_action( 'manage_project_posts_custom_column', array( $this, 'display_thumbnail' ), 10, 1 );
	}

	public function plugin_activation() {
		flush_rewrite_rules();
	}

	public function portfolio_init() {
		$args = array(
			'labels'        => array(
				'name'               => esc_html__( 'Projects', 'themerain' ),
				'singular_name'      => esc_html__( 'Project', 'themerain' ),
				'add_new_item'       => esc_html__( 'Add New Project', 'themerain' ),
				'edit_item'          => esc_html__( 'Edit Project', 'themerain' ),
				'new_item'           => esc_html__( 'New Project', 'themerain' ),
				'view_item'          => esc_html__( 'View Project', 'themerain' ),
				'view_items'         => esc_html__( 'View Projects', 'themerain' ),
				'search_items'       => esc_html__( 'Search Projects', 'themerain' ),
				'not_found'          => esc_html__( 'No projects found', 'themerain' ),
				'not_found_in_trash' => esc_html__( 'No projects found in Trash', 'themerain' ),
				'all_items'          => esc_html__( 'All Projects', 'themerain' )
			),
			'public'        => true,
			'show_in_rest'  => true,
			'menu_position' => 5,
			'menu_icon'     => 'dashicons-portfolio',
			'supports'      => array( 'title', 'editor', 'thumbnail', 'excerpt', 'custom_fields' )
		);

		if ( get_option( 'themerain_portfolio_slug' ) ) {
			$args['rewrite']['slug'] = get_option( 'themerain_portfolio_slug' );
		}

		register_post_type( 'project', $args );

		$tax_args = array(
			'labels'            => array(
				'name'      => esc_html__( 'Project Categories', 'themerain' ),
				'menu_name' => esc_html__( 'Categories', 'themerain' )
			),
			'public'            => false,
			'show_ui'           => true,
			'show_in_rest'      => true,
			'show_admin_column' => true
		);

		register_taxonomy( 'project-category', 'project', $tax_args );
	}

	public function add_thumbnail_column( $columns ) {
		$column_thumb = array( 'thumbnail' => 'Thumbnail' );
		$columns      = array_slice( $columns, 0, 2, true ) + $column_thumb + array_slice( $columns, 1, NULL, true );

		return $columns;
	}

	public function display_thumbnail( $column ) {
		global $post;

		switch ( $column ) {
			case 'thumbnail' :
				echo get_the_post_thumbnail( $post->ID, array( 35, 35 ) );
				break;
		}
	}

	function portfolio_custom_slug() {
		if ( isset( $_POST['permalink_structure'] ) && isset( $_POST['themerain_portfolio_slug'] ) ) {
			$short_domain = wp_unslash( $_POST['themerain_portfolio_slug'] );
			update_option( 'themerain_portfolio_slug', $short_domain );
		}

		add_settings_section(
			'themerain_permalinks_section',
			'Portfolio',
			array( $this, 'custom_slug_callback' ),
			'permalink'
		);
	}

	function custom_slug_callback() {
		$slug = get_option( 'themerain_portfolio_slug' );
		?>

		<p><?php esc_html_e( 'If you like, you may enter custom slug for your projects here. If you leave this blank the default will be used.', 'themerain' ); ?></p>

		<table class="form-table">
			<tbody>
				<tr>
					<th><label for="themerain_portfolio_slug"><?php esc_html_e( 'Custom slug', 'themerain' ); ?></label></th>
					<td><input type="text" id="themerain_portfolio_slug" name="themerain_portfolio_slug" value="<?php echo esc_attr( $slug ); ?>" class="regular-text code"></td>
				</tr>
			</tbody>
		</table>
		<?php
	}
}

new ThemeRainPortfolio();
